package fr.mael.jiwigo.ui.search.tree;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import fr.mael.jiwigo.ui.search.File;

/**
    Copyright (c) 2010, Mael
    All rights reserved.
    
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of jiwigo nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    
    * @author mael
    * File tree
*/
public class FileTree extends JPanel implements TreeSelectionListener, MouseListener {
    private String path;

    private JTextField fieldPath;

    private DefaultMutableTreeNode root = new DefaultMutableTreeNode("");

    private JTree tree;

    public static void main(String[] args) {
	JFrame frame = new JFrame();
	frame.add(new FileTree("/home/mael/Documents"));

	frame.pack();
	frame.setVisible(true);
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    /**
     * Constructor
     * @param path : the path of the directory where to search for files
     */
    public FileTree(String path) {
	super(new BorderLayout());
	this.path = path;
	fieldPath = new JTextField(path);
	fieldPath.addMouseListener(this);
	tree = new JTree(root);
	tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	tree.addTreeSelectionListener(this);
	tree.setCellRenderer(new MyTreeCellRenderer());
	tree.setRootVisible(false);
	JScrollPane treeView = new JScrollPane(tree);
	Dimension minimumSize = new Dimension(100, 50);
	treeView.setMinimumSize(minimumSize);
	this.add(treeView, BorderLayout.CENTER);
	this.add(fieldPath, BorderLayout.NORTH);
	reload();
    }

    public void reload() {
	root.removeAllChildren();
	createNodes(root);
	((DefaultTreeModel) tree.getModel()).reload();
    }

    private void createNodes(DefaultMutableTreeNode root) {
	DefaultMutableTreeNode fileNode = null;
	File f = new File(path);
	try {
	    File fls[] = f.listFiles();
	    ArrayList<File> filter = new ArrayList<File>();
	    for (File file : fls) {
		if ((file.isDirectory() && !(file.getName().charAt(0) == '.'))
			|| (file.isFile() && ((file.getName().toLowerCase().endsWith(".jpg")
				|| file.getName().toLowerCase().endsWith(".jpeg") || file.getName().toLowerCase()
				.endsWith(".png"))))) {
		    filter.add(file);
		}
	    }
	    for (File file : filter) {
		fileNode = new DefaultMutableTreeNode(file);
		root.add(fileNode);
		recursiveNodeCreation(fileNode, file);

	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

    }

    /**
     * recursive method for the creation of the nodes
     * @param categoryNode
     * @param category
     */
    private void recursiveNodeCreation(DefaultMutableTreeNode categoryNode, File file) {
	if (file.isDirectory() && !(file.getName().charAt(0) == '.')) {
	    File fls[] = file.listFiles();
	    ArrayList<File> filter = new ArrayList<File>();
	    for (File fich : fls) {
		if (!(fich.getName().charAt(0) == '.')
			&& fich.isDirectory()
			|| (fich.isFile() && ((fich.getName().toLowerCase().endsWith(".jpg")
				|| fich.getName().toLowerCase().endsWith(".jpeg") || fich.getName().toLowerCase()
				.endsWith(".png"))))) {
		    filter.add(fich);
		}
	    }
	    for (File fich : filter) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(fich);
		categoryNode.add(node);
		recursiveNodeCreation(node, fich);
	    }
	}
    }

    @Override
    public void valueChanged(TreeSelectionEvent arg0) {

    }

    @Override
    public void mouseClicked(MouseEvent arg0) {
	if (arg0.getSource().equals(fieldPath)) {
	    JFileChooser chooser = new JFileChooser();
	    chooser.setCurrentDirectory(new java.io.File(path));
	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    chooser.setAcceptAllFileFilterUsed(false);
	    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
		path = chooser.getSelectedFile().getAbsolutePath();
		fieldPath.setText(path);
		reload();
	    }
	}
    }

    @Override
    public void mouseEntered(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    public JTree getTree() {
	return tree;
    }

}
